@@ -195,10 +195,10 @@ static ulong _kl_pg_table_deref_s390x(unsigned long vaddr, unsigned long table)
 	readmem(VADDR, table + offset, &entry, sizeof(entry));
 	/*
 	 * Check if the page table entry could be read and doesn't have
-	 * any of the reserved bits set.
+	 * the reserved bit set.
 	 * Check if the page table entry has the invalid bit set.
 	 */
-	if (entry &  (_PAGE_CO | _PAGE_ZERO | _PAGE_INVALID)) {
+	if (entry &  (_PAGE_ZERO | _PAGE_INVALID)) {
 		ERRMSG("Invalid page table entry.\n");
 		return 0;
 	}
@@ -247,6 +247,11 @@ vtop_s390x(unsigned long vaddr)
 			return NOT_PADDR;
 		}
 		table = entry & _REGION_ENTRY_ORIGIN;
+		if ((entry & _REGION_ENTRY_LARGE) && (level == 1)) {
+			table &= ~0x7fffffffUL;
+			paddr = table + (vaddr & 0x7fffffffUL);
+			return paddr;
+		}
 		len = RSG_TABLE_LENGTH(entry);
 		level--;
 	}
@@ -257,6 +262,7 @@ vtop_s390x(unsigned long vaddr)
 	 * if no, then get the page table entry using PX index.
 	 */
 	if (entry & _SEGMENT_ENTRY_LARGE) {
+		table &= ~_PAGE_BYTE_INDEX_MASK;
 		paddr = table + (vaddr &  _PAGE_BYTE_INDEX_MASK);
 	} else {
 		entry = _kl_pg_table_deref_s390x(vaddr,
