@@ -751,7 +751,7 @@ sadump_initialize_bitmap_memory(void)
 {
 	struct sadump_header *sh = si->sh_memory;
 	struct dump_bitmap *bmp;
-	unsigned long dumpable_bitmap_offset, dumpable_bitmap_len;
+	unsigned long dumpable_bitmap_offset;
 	unsigned long long section, max_section, pfn;
 	unsigned long long *block_table;
 
@@ -759,8 +759,6 @@ sadump_initialize_bitmap_memory(void)
 		si->sub_hdr_offset +
 		sh->block_size * (sh->sub_hdr_size + sh->bitmap_blocks);
 
-	dumpable_bitmap_len = sh->block_size * sh->dumpable_bitmap_blocks;
-
 	bmp = malloc(sizeof(struct dump_bitmap));
 	if (bmp == NULL) {
 		ERRMSG("Can't allocate memory for the memory-bitmap. %s\n",
@@ -949,11 +947,10 @@ failed:
 #endif /* __x86_64__ */
 
 int
-readpmem_sadump(unsigned long long paddr, void *bufptr, size_t size)
+readpage_sadump(unsigned long long paddr, void *bufptr)
 {
 	unsigned long long pfn, block, whole_offset, perdisk_offset;
 	ulong page_offset;
-	char buf[info->page_size];
 	int fd_memory;
 
 	if (si->kdump_backed_up &&
@@ -965,12 +962,12 @@ readpmem_sadump(unsigned long long paddr, void *bufptr, size_t size)
 	page_offset = paddr % info->page_size;
 
 	if (pfn >= si->sh_memory->max_mapnr)
-		goto error;
+		return FALSE;
 
 	if (!is_dumpable(info->bitmap_memory, pfn)) {
 		ERRMSG("pfn(%llx) is excluded from %s.\n", pfn,
 		       info->name_memory);
-		goto error;
+		return FALSE;
 	}
 
 	block = pfn_to_block(pfn);
@@ -980,7 +977,7 @@ readpmem_sadump(unsigned long long paddr, void *bufptr, size_t size)
 		int diskid;
 
 		if (!lookup_diskset(whole_offset, &diskid, &perdisk_offset))
-			goto error;
+			return FALSE;
 
 		fd_memory = si->diskset_info[diskid].fd_memory;
 		perdisk_offset += si->diskset_info[diskid].data_offset;
@@ -992,19 +989,12 @@ readpmem_sadump(unsigned long long paddr, void *bufptr, size_t size)
 	}
 
 	if (lseek(fd_memory, perdisk_offset, SEEK_SET) < 0)
-		goto error;
+		return FALSE;
 
-	if (read(fd_memory, buf, sizeof(buf)) != sizeof(buf))
-		goto error;
+	if (read(fd_memory, bufptr, info->page_size) != info->page_size)
+		return FALSE;
 
-	memcpy(bufptr, buf + page_offset, size);
-
-	return size;
-
-error:
-	DEBUG_MSG("type_addr: %d, addr:%llx, size:%zd\n", PADDR, paddr, size);
-
-	return FALSE;
+	return TRUE;
 }
 
 int
